import { RequestHandler, Router } from 'express';
import { CreateCard, ListCards, ReadCard } from '../../functions/card';
import { Services } from '../../app';
import { CheckPermissions, getRequestId, ValidateSchema } from './middleware';
import { CreateCardSchema, ReadCardSchema } from './schema/card';

function CreateHandler(services: Services): RequestHandler {
  const createCard = CreateCard(services);

  return async (req, res, next) => {
    try {
      const card = await createCard(getRequestId(req), req.body);
      res.status(200).send(card);
    } catch (error) {
      next({ status: 409, message: 'Card Already Exists' });
    }
  };
}

function ReadHandler(services: Services): RequestHandler {
  const readCard = ReadCard(services);

  return async (req, res, next) => {
    try {
      const card = await readCard(getRequestId(req), req.params.uuid);
      res.status(200).send(card);
    } catch (error) {
      next({ status: 404, message: 'Card Not Found' });
    }
  };
}

function ListHandler(services: Services): RequestHandler {
  const listCards = ListCards(services);

  return async (req, res, next) => {
    try {
      const cards = await listCards(getRequestId(req));
      res.status(200).send(cards);
    } catch (error) {
      next({ message: 'Unknown Error' });
    }
  };
}

export function Routes(services: Services) {
  const router = Router();

  router.post(
    '/create',
    CheckPermissions(),
    CreateCardSchema(),
    ValidateSchema(),
    CreateHandler(services),
  );
  router.get(
    '/read/:uuid',
    CheckPermissions(),
    ReadCardSchema(),
    ValidateSchema(),
    ReadHandler(services),
  );
  router.get('/list', CheckPermissions(), ListHandler(services));

  return router;
}
