import express, { Express } from 'express';
import cors from 'cors';
import session from 'express-session';
import MongoStore from 'connect-mongo';
import { randomUUID } from 'crypto';
import * as http from 'http';

import { Routes } from './router';
import { RequestId, ErrorHandler, NotFoundHandler } from './middleware';
import { Services } from '../../app';
import { MongoConfig, ServerConfig } from '../../config';

class Server {
  private app: Express;
  private server: http.Server;
  private config: ServerConfig;

  constructor(
    config: ServerConfig,
    mongoConfig: MongoConfig,
    services: Services,
  ) {
    this.config = config;

    this.app = express();
    this.app.disable('x-powered-by');
    this.app.use(express.json());
    this.app.use(
      cors({
        maxAge: 86400,
        credentials: true,
        origin: this.config.origin,
      }),
    );
    this.app.use(
      session({
        proxy: process.env.NODE_ENV === 'production',
        secret: randomUUID(),
        cookie: {
          maxAge: 1000 * 3600 * 24,
          sameSite: 'strict',
          secure: process.env.NODE_ENV === 'production',
        },
        store: MongoStore.create({
          mongoUrl: mongoConfig.uri,
          dbName: mongoConfig.dbName,
          collectionName: 'sessions',
          touchAfter: 3600,
        }),
        resave: false,
        saveUninitialized: false,
      }),
    );
    this.app.use(RequestId());
    this.app.use(Routes(services));
    this.app.use(NotFoundHandler());
    this.app.use(ErrorHandler());
  }

  async start(): Promise<void> {
    return new Promise<void>((resolve, reject) => {
      this.server = this.app.listen(this.config.port, resolve);
    });
  }

  async close(): Promise<void> {
    return new Promise<void>((resolve, reject) => {
      this.server.close((error) => {
        if (error) reject(error);
        resolve();
      });
    });
  }
}

export default Server;
