import { RequestHandler, Router } from 'express';
import { Services } from '../../app';
import {
  CreateUser,
  ReadUser,
  LoginUser,
  UpdateUser,
  ListUsers,
  AddCardToUser,
} from '../../functions/user';
import {
  LoginUserSchema,
  CreateUserSchema,
  ReadUserSchema,
  LogoutUserSchema,
  UpdateUserSchema,
  ListUsersSchema,
  AddCardToUserSchema,
  RemoveCardFromUserSchema,
} from './schema/user';
import { CheckPermissions, getRequestId, ValidateSchema } from './middleware';

function LoginUserHandler(services: Services): RequestHandler {
  const login = LoginUser(services);

  return async (req, res, next) => {
    try {
      const user = await login(getRequestId(req), req.body);
      user ? (req.session.user = user) : (req.session.user = null);
      res.status(200).send(user);
    } catch (error) {
      next({ status: 401, message: 'wrong username or password' });
    }
  };
}

function LogoutUserHandler(services: Services): RequestHandler {
  return async (req, res, next) => {
    if (req.session.user) {
      req.session.user = null;
      res.status(204).send();
    } else {
      next({ status: 401, message: 'Not Logged In' });
    }
  };
}

function CreateUserHandler(services: Services): RequestHandler {
  const createUser = CreateUser(services);

  return async (req, res, next) => {
    try {
      const user = await createUser(getRequestId(req), req.body);
      res.status(201).send(user);
    } catch (error) {
      next({ status: 409, message: 'User Already Exists' });
    }
  };
}

function ReadUserHandler(services: Services): RequestHandler {
  const readUser = ReadUser(services);

  return async (req, res, next) => {
    try {
      const user = await readUser(getRequestId(req), req.params.uuid);
      res.status(200).send(user);
    } catch (error) {
      next({ status: 404, message: 'User Not Found' });
    }
  };
}

function UpdateUserHandler(services: Services): RequestHandler {
  const updateUser = UpdateUser(services);

  return async (req, res, next) => {
    try {
      const user = await updateUser(
        getRequestId(req),
        req.params.uuid,
        req.body,
      );
      res.status(200).send(user);
    } catch (error) {
      next({ status: 404, message: 'User Not Found' });
    }
  };
}

function ListUserHandler(services: Services): RequestHandler {
  const listUsers = ListUsers(services);

  return async (req, res, next) => {
    try {
      const users = await listUsers(getRequestId(req));
      res.status(200).send(users);
    } catch (error) {
      next({ message: 'Unknown Error' });
    }
  };
}

function AddCardToUserHandler(services: Services): RequestHandler {
  const addCardToUser = AddCardToUser(services);

  return async (req, res, next) => {
    try {
      const user = await addCardToUser(
        getRequestId(req),
        req.params.uuid,
        req.body.cardUuid,
      );
      res.status(200).send(user);
    } catch (error) {
      next({ message: 'Unknown Error' });
    }
  };
}

export function Routes(services: Services) {
  const router = Router();

  //User
  router.post(
    '/login',
    LoginUserSchema(),
    ValidateSchema(),
    LoginUserHandler(services),
  );
  router.post(
    '/logout',
    LogoutUserSchema(),
    ValidateSchema(),
    LogoutUserHandler(services),
  );
  router.get(
    '/read/:uuid',
    CheckPermissions(),
    ReadUserSchema(),
    ValidateSchema(),
    ReadUserHandler(services),
  );
  router.post(
    '/create',
    CheckPermissions(),
    CreateUserSchema(),
    ValidateSchema(),
    CreateUserHandler(services),
  );
  router.patch(
    '/update/:uuid',
    CheckPermissions(),
    UpdateUserSchema(),
    ValidateSchema(),
    UpdateUserHandler(services),
  );
  router.get(
    '/list',
    CheckPermissions(),
    ListUsersSchema(),
    ValidateSchema(),
    ListUserHandler(services),
  );

  //Alteration
  router.post(
    '/addCard/:uuid',
    CheckPermissions(),
    AddCardToUserSchema(),
    ValidateSchema(),
    AddCardToUserHandler(services),
  );
  router.post(
    '/removeCard/:uuid',
    CheckPermissions(),
    RemoveCardFromUserSchema(),
    ValidateSchema(),
    //RemoveCardFromUserHandler(),
  );

  return router;
}
