import { Collection, Db } from 'mongodb';
import { Card, CardInfo, LoginUserBody, UpdateUserBody, UserInfo } from '@core';
import { User, UserWithPassword } from '../../entities/user';
import { log } from '../../functions/logger';
import { getHashedPassword } from '../../functions/password';

class UserModel {
  private collection: Collection<User>;

  constructor(db: Db) {
    this.collection = db.collection<User>('users');
  }

  public async login(tracker: string, userInfo: LoginUserBody): Promise<User> {
    const checkUser = await this.collection.findOne({
      username: userInfo.username,
    });
    if (!checkUser) {
      log(tracker, 'User Not Found');
      throw new Error();
    }

    const userDocument = await this.collection.findOne({
      uuid: checkUser.uuid,
      password: getHashedPassword(checkUser.uuid, userInfo.password),
    });
    if (!userDocument) {
      log(tracker, 'Wrong Password');
      throw new Error();
    }

    const user = new User(userDocument);
    log(tracker, 'LOG IN', user);
    return user;
  }

  public async create(
    tracker: string,
    userInfo: UserWithPassword,
  ): Promise<User> {
    const checkUser = await this.collection.findOne({
      username: userInfo.username,
    });
    if (checkUser) {
      log(tracker, 'User Already Exists');
      throw new Error();
    }

    await this.collection.insertOne(userInfo);

    const user = await this.read(tracker, userInfo.uuid);
    log(tracker, 'CREATE USER', user);
    return user;
  }

  public async read(tracker: string, uuid: string): Promise<User> {
    const [userDocument] = await this.collection.aggregate<UserInfo>([
      {
        $match:{
          uuid
        }
      },
      {
        $lookup: {
          from: "cards",
          localField: "cards",
          foreignField: "_id",
          as: "cards"
        }
      }
    ]).toArray();
    console.log(JSON.stringify(userDocument));
    if (!userDocument) {
      log(tracker, 'User Not Found');
      throw new Error();
    }

    const user = new User(userDocument);
    log(tracker, 'READ USER', user);
    return user;
  }

  public async update(
    tracker: string,
    uuid: string,
    userInfo: UpdateUserBody,
  ): Promise<User> {
    const checkUser = await this.collection.findOne({ uuid });
    if (!checkUser) {
      log(tracker, 'User Does Not Exist');
      throw new Error();
    }

    if (userInfo.password)
      userInfo.password = getHashedPassword(uuid, userInfo.password);

    await this.collection.updateOne(
      { uuid },
      {
        $set: {
          ...userInfo,
          updatedAt: new Date(),
        },
      },
    );

    const user = await this.read(tracker, uuid);
    log(tracker, 'UPDATE USER', user);
    return user;
  }

  public async list(tracker: string): Promise<User[]> {
    const userCollection = await this.collection.find().toArray();
    log(tracker, 'LIST USERS');
    return userCollection.map((userDocument) => new User(userDocument));
  }

  public async addCard(
    tracker: string,
    uuid: string,
    card: Card,
  ): Promise<User> {
    const checkUser = await this.collection.findOne({ uuid });
    if (!checkUser) {
      log(tracker, 'User Does Not Exist');
      throw new Error();
    }

    await this.collection.updateOne(
      { uuid },
      {
        $set: {
          updatedAt: new Date(),
        },
        $addToSet: {
          cards: card._id,
        },
      },
    );

    const user = await this.read(tracker, uuid);
    log(tracker, 'ADD CARD TO USER', user, card);
    return user;
  }
}

export default UserModel;
